#include "Conflict.h"

/* IP library functions */
/* -------------------- */

short *IPToShortArray(const IP *ip_obj, short ip_array[])
{
	int i;
	unsigned ip = ip_obj->ip;

	assert(sizeof(ip_obj->ip) == BYTE_SIZE_UNSIGNED);

	for(i = IP_SIZE - 2; i > -1; i--){
		ip_array[i] = (ip & BYTE_SET_ON);
		ip >>= BYTE_SIZE;
	}

	ip_array[IP_SIZE - 1] = ip_obj->mask;

	return ip_array;
}

IP *shortArrayToIP(const short ip_array[], IP *ip_obj)
{
	int i;

	assert(sizeof(ip_obj->ip) == BYTE_SIZE_UNSIGNED);

	ip_obj->ip = 0;

	for(i = 0; i < IP_SIZE - 1; i++){
		ip_obj->ip <<= BYTE_SIZE;
		ip_obj->ip |= ip_array[i];
	}

	ip_obj->mask = ip_array[IP_SIZE - 1];

	return ip_obj;
}

char *IPToStr(const IP *ip_obj, char ip_str[])
{
	short ip_array[IP_SIZE];

	IPToShortArray(ip_obj, ip_array);

	sprintf(ip_str, "%u.%u.%u.%u/%u",
		ip_array[0], ip_array[1], ip_array[2],
		ip_array[3], ip_array[4]);

	return ip_str;
}

IP *strToIP(const char ip_str[], IP *ip_obj)
{
	int i;
	unsigned ip_token;
	char *str_ptr, *str_dup = strdup(ip_str);

	assert(sizeof(ip_obj->ip) == BYTE_SIZE_UNSIGNED);

	ip_obj->ip = 0;

	str_ptr = strtok(str_dup, ".");
	
	for(i = 0; i < IP_SIZE - 1; i++){
		sscanf(str_ptr, "%u", &ip_token);
		str_ptr = strtok(NULL, "./");
		ip_obj->ip <<= BYTE_SIZE;
		ip_obj->ip |= ip_token;
	}

	sscanf(str_ptr, "%u", &(ip_obj->mask));
	free(str_dup);

	return ip_obj;
}


/* Conflict library functions */
/* -------------------------- */

Conflict* createConflict(const IP *ip_obj, const unsigned AS,
						 List *previous_AS,
						 const time_t conflict_time,
						 const time_t previous_time,
						 const Security security_level)
{
	Conflict* conflict = (Conflict *)malloc(sizeof(Conflict));
	if(conflict){
		if (ip_obj){
			(conflict->ip).ip		= ip_obj->ip;
			(conflict->ip).mask		= ip_obj->mask;
		}
		conflict->AS				= AS;
		conflict->previous_AS		= previous_AS;
		conflict->conflict_time		= conflict_time;
		conflict->previous_time		= previous_time;
		conflict->security_level	= security_level;
	}

	return conflict;
}

void freeConflict(Conflict* conflict)
{
	if (conflict){
		freeList(conflict->previous_AS);
		free(conflict);
	}
}

void fprintConflict(FILE* output, const Conflict* conflict)
{
	char ip_str[IP_STR_MAX_SIZE];

	if(!conflict)
		return;

	fprintf(output, "IP:\t%s\nAS:\t%d\nDate:\t%s",
			IPToStr(&(conflict->ip), ip_str), conflict->AS,
			ctime(&(conflict->conflict_time)));

	fprintf(output, "The following AS ");

	if (conflict->security_level == SECURITY_MIN)
		fprintf(output, "are registered with the same prefix:\n");
	else
		fprintf(output, "were in use since %s",
				ctime(&(conflict->previous_time)));

	fprintList(output, conflict->previous_AS);

	fprintf(output, "Security level: ");

	switch(conflict->security_level){
	case SECURITY_MIN:
		fprintf(output, "minimum\n\n");
		break;
	case SECURITY_MED:
		fprintf(output, "medium\n\n");
		break;
	case SECURITY_MAX:
		fprintf(output, "maximal\n\n");
	}
}
