#ifndef _CONFLICT_H
#define _CONFLICT_H

#include "List.h"
#include <string.h>
#include <time.h>
#include <assert.h>


/* IP structure */
/* ------------ */

typedef struct
{
	unsigned ip;
	short mask;
} IP;


/* IP macro definitions */
/* -------------------- */

#define IP_SIZE				5	// IP format: xxx.xxx.xxx.xxx/xx
#define IP_STR_MAX_SIZE		19	// including terminating '\0'
#define BYTE_SIZE			8	// 1 Byte = 8 bit
#define BYTE_SIZE_UNSIGNED	4	// size of unsigned (Bytes)
#define BYTE_SET_ON			255	// 255 = 11111111 (radix 2)


/* IP library functions */
/* -------------------- */

short *IPToShortArray(const IP *ip_obj, short ip_array[]);

IP *shortArrayToIP(const short ip_array[], IP *ip_obj);

char *IPToStr(const IP *ip_obj, char ip_str[]);

IP *strToIP(const char ip_str[], IP *ip_obj);


/* Security levels */
/* --------------- */

typedef enum {SECURITY_MIN,
			  SECURITY_MED,
			  SECURITY_MAX
} Security;

#define DEFAULT_SECURITY_MODE SECURITY_MED

/* Conflict structure */
/* ------------------ */

typedef struct Conflict
{
	IP ip;
	unsigned AS;
	List *previous_AS;
	time_t conflict_time;
	time_t previous_time;
	Security security_level;

} Conflict;


/* Conflict library functions */
/* -------------------------- */

// Allocate memory and initialize a conflict.
// Return: a pointer to the new conflict.
// Note:	1) The function uses a copy of the given IP.
//			2) Therefore, When unknown, the given IP may be NULL.
//			3) The function DOES NOT copy the given list;
//			   When freed, the object destroys the list.
Conflict* createConflict(const IP *ip_obj, const unsigned AS,
						 List *previous_AS,
						 const time_t conflict_time,
						 const time_t previous_time,
						 const Security security_level);

// Free all the memory that was allocated to the conflict.
// Note: the internal list is destryed.
void freeConflict(Conflict* conflict);

// Print a conflict to an output file.
void fprintConflict(FILE* output, const Conflict* conflict);



#endif /* _CONFLICT_H */
