#ifndef _GLOBAL_DS_H 
#define _GLOBAL_DS_H

#include <signal.h>
#include "Queue.h"

#ifdef _WIN32	// Windows compatibility

#include "pthread.h"

#define SIGUSR1 SIGTERM

int pthread_kill(pthread_t thread, int sig);

#else

#include <pthread.h>

#endif

/* Errors definitions */
/* ------------------ */

typedef enum {NOT_ERROR,
			  ERROR_MEMORY_ALLOCATION,
			  ERROR_OPEN_FILE,
			  ERROR_CLOSE_FILE,
			  ERROR_WRITE_FILE,
			  ERROR_CREATE_THREAD,
			  ERROR_INITWINSOCKDLL,
			  ERROR_UNKNOWN_HOST,
			  ERROR_OPEN_SOCKET,
			  ERROR_CLOSE_SOCKET,
			  ERROR_CONNECTION_FAILED,
			  ERROR_SEND_REQUEST,
			  GENERAL_ERROR
} Error;


/* Macro definitions */
/* ----------------- */

// file names
#define CONFLICTS_LOG_FILE_NAME			"conflicts_log.txt"
#define ZIPPED_FILE_NAME				"routing_table.gz"
#define EXTRACTED_FILE_NAME				"routing_table"
#define ASCII_FILE_NAME					"routing_table.txt"

// web connection
#define HOST							"wwwproxy.ac.il"
#define PORT							8080
#define BASE_URL "http://data.ris.ripe.net/rrc00/0000.00/bview.00000000.0000.gz"

// default parameters
#define DEFAULT_SUSPICION_MAX_TIME		172800	// =2*24*60*60 (sec)
#define DEFAULT_CONFLICTS_QUEUE_SIZE	50000
#define DEFAULT_IP_TRACKING_QUEUE_SIZE	20
#define DEFAULT_DOWNLOAD_CYCLE			28800	// =8*60*60 (sec)


/* Global data structure */
/* --------------------- */

time_t startup_time; 

unsigned long conflicts_min_counter;
unsigned long conflicts_med_counter;
unsigned long conflicts_max_counter;

unsigned suspicion_max_time;

unsigned conflicts_queue_size;
unsigned ip_tracking_queue_size;

unsigned download_cycle;

Security security_mode;

Trie *trie;

Queue* conflicts_queue;

pthread_t downloader_thread;

pthread_mutex_t trie_locker;

pthread_cond_t startup_event;


/* library functions */
/* ----------------- */

Error initializeDS();

void terminateDS(int n);

void registerConflict(Conflict* conflict);


/* external functions */
/* ------------------ */

extern void* threadCyclicDownload(void*);


#endif	/* _GLOBAL_DS_H */
