#ifndef _LIST_H 
#define _LIST_H

#include <stdio.h>
#include <stdlib.h>

/* linked list structure */
/* --------------------- */

typedef struct List{

    int data;
    struct List *next;

} List;

/* library functions */
/* ----------------- */

// Allocate memory for a list.
// Return: pointer to the new list.
List *createList(int data, List *next);

// Delete all elements in a list and free allocated memory.
void freeList(List *head);

// Insert a new data node to the head of a given list.
void insertList(List **head, const int data);

// Allocate memory and copy the elemnts of the given list.
// Return: a pointer to the head of the new list.
// Note:	1) the copied list is in reversed order.
//			2) the copied list should be freed.
List *copyList(List* original);

// Print all the data contained in the given list.
// Note: the output is terminated by a '\n' character.
void fprintList(FILE* output, const List *head);

// Find a 'data' in a given list.
// Return: the node that contains the given 'data',
// or NULL in case the list does not contain that 'data'.
List *findList(List *head, const int data);

// Return the list's length.
int lengthList(const List *head);


#endif	/* _LIST_H */
