#include "Queue.h"

Queue* createQueue(const unsigned size)
{
	Queue* queue = (Queue *)calloc(1, sizeof(Queue));

	if (queue){
		queue->conflicts =
				(Conflict**)calloc(size, sizeof(Conflict*));
		queue->size = size;
	}

	return queue;
}


void freeQueue(Queue* queue)
{
	if (queue){
		unsigned i;
		for(i = queue->head; i < queue->tail; i++)
			freeConflict(queue->conflicts[i % queue->size]);
		free(queue->conflicts);
		free(queue);
	}
}


Conflict* push(Queue* queue, Conflict* conflict)
{
	Conflict* oldest_conflict = NULL;

	assert(queue);

	if (isFull(queue))
		oldest_conflict = pop(queue);
	queue->conflicts[queue->tail % queue->size] = conflict;
	(queue->tail)++;

	return oldest_conflict;
}


Conflict* pop(Queue* queue)
{
	Conflict* conflict = NULL;

	if(!isEmpty(queue)){
		conflict = queue->conflicts[queue->head];
		queue->conflicts[queue->head] = NULL;
		(queue->head)++;
		if (queue->head == queue->size){
			queue->head = 0;
			queue->tail -= queue->size;
		}
	}

	return conflict;
}


int queueSize(const Queue* queue)
{
	if (queue)
		return (queue->tail - queue->head);
	return -1;
}


bool isEmpty(const Queue* queue)
{
	return (queue == NULL || (queue->head == queue->tail));
}


bool isFull(const Queue* queue)
{
	return (queue != NULL &&
			(queue->tail - queue->head == queue->size));
}


void fprintQueue(FILE* output, const Queue* queue)
{
	if(queue){
		unsigned i;
		for(i = queue->head; i < queue->tail; i++)
			fprintConflict(output,
						   queue->conflicts[i % queue->size]);
	}
}


void fflushConflictsQueue(FILE *output, Queue* queue)
{
	int i, flush_amount = 2 * queueSize(queue) / 3;

	for (i = 0; i < flush_amount; i++){
		Conflict *conflict = pop(queue);
		fprintConflict(output, conflict);
		freeConflict(conflict);
	}
}
