#ifndef _QUEUE_H
#define _QUEUE_H

#include "Trie.h"

/* Queue structure */
/* --------------- */

typedef struct Queue
{
    Conflict** conflicts;
	unsigned head;
	unsigned tail;
	unsigned size;

} Queue;

/* Queue library functions */
/* ----------------------- */

// Allocate memory and initialize an empty queue.
// The queue size is limited by the 'size' parameter.
// Return: a pointer to the new queue.
Queue* createQueue(const unsigned size);

// Free all the memory that was allocated to the queue.
void freeQueue(Queue* queue);

// Push a conflict to the queue.
// Return:	the oldest conflict in the queue, if the queue is full;
//			otherwise, return 'NULL'. 
// Note:	1) it is assumed that the queue is valid (not NULL).
//			2) in case a conflict is returned, its allocated
//			   memory should be freed seperately.
Conflict* push(Queue* queue, Conflict* conflict);

// Pop the oldest conflict in the queue.
// Return:	the oldest conflict, or NULL if the queue is empty.
Conflict* pop(Queue* queue);

// Return: the current number of conflicts in the queue,
// or -1 if the queue is NULL.
int queueSize(const Queue* queue);

// Return: 'true' if the queue is empty or if it is NULL.
bool isEmpty(const Queue* queue);

// Return: 'true' if the queue is full.
bool isFull(const Queue* queue);

// Print all the conflicts in a queue to an output file.
// It is assumed, that the file is already open for writing.
void fprintQueue(FILE* output, const Queue* queue);

// For each of about 2/3 of the oldest conflicts in the queue
// do the following:
// pop out, print to the output file, and free the conflict.
// Note:	1) It is assumed, that the file is already
//			   open for writing.
//			2) it is recomended to use this funcion whenever
//			   the queue is full.
void fflushConflictsQueue(FILE *output, Queue* queue);


#endif /* _QUEUE_H */
