#ifndef _TRIE_H
#define _TRIE_H

#include "Conflict.h"

/* Boolean values */
/* -------------- */

typedef enum {false, true} bool;


/* Trie tree macro definitions */
/* --------------------------- */

#define CHILDREN_ARRAY_SIZE	256	// 256 = 2^8
#define MASK_ARRAY_SIZE		BYTE_SIZE


/* ASList structure */
/* ---------------- */

typedef struct ASList
{
	List *AS_list;
	time_t AS_insert_time;

} ASList;

// Note: all the ASList functions are defined as 'static'.


/* Trie tree structure */
/* ------------------- */

typedef struct Trie{

    struct Trie** children;
	ASList **AS_array;

} Trie;


/* Trie tree library functions */
/* --------------------------- */

// Allocate memory and initialize an empty trie tree.
// Return: a pointer to the new trie tree.
Trie* createTrie();

// Free all the memory that was allocated to the trie tree.
void freeTrie(Trie* trie);

// Insert an AS to a trie tree.
// Input:	a non-NULL pointer to a trie tree, a valid IP,
//			an AS that belongs to the IP, and the build up time.
// Return:	'NULL' if an ownership conflict was not detected;
//			a new conflict otherwise.
// Note:	rhe returned Conflict object should be freed.
Conflict *insertPrefix(Trie* trie,
					   const IP *ip, const unsigned AS,
					   const time_t build_time);

// Find the AS of a given IP.
// Return:	a pointer to a list of all the AS that belong to
// the given IP (may be a NULL pointer, if no such AS exist).
// Note: this function returns a pointer to the list in the
// given trie tree, and does not create a copy of that list.
List* findAS(Trie* trie, const IP *ip);

// Print all the AS in a trie tree to an output file.
void fprintTrie(FILE* output, const Trie* trie);


/* external global variables */
/* ------------------------- */

extern unsigned suspicion_max_time;
extern Security security_mode;


#endif /* _TRIE_H */
